// Copyright (C) 1998 Logi Ragnarsson

package is.logi.crypto.test;
import is.logi.crypto.*;
import is.logi.crypto.random.*;

import java.util.Random;
import java.security.SecureRandom;
import java.io.*;

/**
 * This application tests the various random number generation classes.
 *
 * @author <a href="http://www.hi.is/~logir/">Logi Ragnarsson</a>
 * (<a href="mailto:logir@hi.is">logir@hi.is</a>)
 *
 * @see is.logi.crypto.keys.Key
 */
public class TestRandom extends Crypto{

    private TestRandom(){}
    
    /**
     * Anything written to instances of this class
     * is counted and discarded.
     */
    private static class BitBucket extends FilterOutputStream{
        
        private int count=0;
        
        public BitBucket(){
            super(null);
        }
        
        public void close(){
        }
        
        public void flush(){
        }
        
        public void write(byte[] buf){
            count += buf.length;
        }
        
        public void write(byte[] buf, int off, int len){
            count += len;
        }
        
        public void write(int x){
            count += 1;
        }
        
        public int getCount(){
            return count;
        }
        
    }
    
    private static void printTime(long t){
        System.out.println("Time: " + (t/1000) + "." + (t%1000) + " s");
    }
    
    private static final int K=4;
    
    private static void test(Random r) throws Exception{
        byte[] bits = new byte[1024*K];
        System.out.println("Generating "+K+" KB of random bits");
        long start = System.currentTimeMillis();
        r.nextBytes(bits);
        long end = System.currentTimeMillis();
        printTime(end-start);
        
        
        // Apparently, compression is not a good randomness test, since the
        // output of java.util.Random can't be compressed eny better than the
        // proper PRNGs. I need to find something better.
        
        BitBucket bitBucket = new BitBucket();
        OutputStream out = new java.util.zip.GZIPOutputStream(bitBucket,K*1024);
        out.write(bits);
        out.close();
        
        int compressed = bitBucket.getCount()-23;
        double ratio = (double)compressed / (K*1024);
        System.out.println("Compressed to " + compressed + " bytes, ratio: " + ratio);
        
        System.out.println();
    }
    
    public static void main(String[] arg){
        long start,end;
        Random r;
        
        try{
            System.out.println("Initializing java.util.Random");
            start = System.currentTimeMillis();
            r = new Random();
            end = System.currentTimeMillis();
            printTime(end-start);
            test(r);
        } catch (Throwable e){
            e.printStackTrace();
        }
        
        try{
            System.out.println("Initializing java.security.SecureRandom");
            start = System.currentTimeMillis();
            r = new SecureRandom();
            end = System.currentTimeMillis();
            printTime(end-start);
            test(r);
        } catch (Throwable e){
            e.printStackTrace();
        }
        
        try{
            System.out.println("Initializing is.logi.crypto.random.RandomSpinner");
            start = System.currentTimeMillis();
            r = new RandomSpinner();
            end = System.currentTimeMillis();
            printTime(end-start);
            test(r);
        } catch (Throwable e){
            e.printStackTrace();
        }
        
        try{
            System.out.println("Initializing is.logi.crypto.random.PureSpinner");
            start = System.currentTimeMillis();
            r = new PureSpinner();
            end = System.currentTimeMillis();
            printTime(end-start);
            test(r);
        } catch (Throwable e){
            e.printStackTrace();
        }
        
    }
    
}